﻿using System;
using System.ComponentModel;
using System.ServiceModel.DomainServices.Client;

namespace DeleteFromEntitySet
{
	public partial class EntityCollectionViewBase<TEntity> : IEditableCollectionView where TEntity : Entity, new()
	{
		/// <summary>Adds a new item to the underlying collection.</summary>
		/// <returns>The new item that is added to the collection.</returns>
		public object AddNew()
		{
			if (this.IsAddingNew)
			{
				this.CommitNew();
			}

			var addingEventArgs = new CancelableEntityEventArgs<TEntity>();
			this.OnAddingNew(addingEventArgs);

			if (addingEventArgs.Cancel)
			{
				return null;
			}

			this.CurrentAddItem = addingEventArgs.Entity ?? new TEntity();
			this.delegates.Add(this.CurrentAddItem);
			((IEditableObject) this.CurrentAddItem).BeginEdit();

			var addedEventArgs = new EntityEventArgs<TEntity>(this.CurrentAddItem);
			this.OnAddedNew(addedEventArgs);

			return this.CurrentAddItem;
		}

		/// <summary>Ends the add transaction and saves the pending new item.</summary>
		public void CommitNew()
		{
			if (!this.IsAddingNew)
			{
				return;
			}

			var committingNewEventArgs = new CancelEventArgs();
			this.OnCommittingNew(committingNewEventArgs);

			if (committingNewEventArgs.Cancel)
			{
				return;
			}

			((IEditableObject) this.CurrentAddItem).EndEdit();
			this.CurrentAddItem = null;

			var committedNewEventArgs = new EntityEventArgs<TEntity>(this.CurrentAddItem);
			this.OnCommittedNew(committedNewEventArgs);
		}

		/// <summary>Ends the add transaction and discards the pending new item.</summary>
		public void CancelNew()
		{
			if (!this.IsAddingNew)
			{
				return;
			}

			var cancellingNewEventArgs = new CancelEventArgs();
			this.OnCancellingNew(cancellingNewEventArgs);

			if (cancellingNewEventArgs.Cancel)
			{
				return;
			}

			var currentAddItem = this.CurrentAddItem;

			((IEditableObject) this.CurrentAddItem).CancelEdit();
			this.delegates.Remove(this.CurrentAddItem);
			this.CurrentAddItem = null;

			var cancelledNewEventArgs = new EntityEventArgs<TEntity>(currentAddItem);
			this.OnCancelledNew(cancelledNewEventArgs);
		}

		/// <summary>Removes the specified item from the collection.</summary>
		/// <param name="item">The item to remove.</param>
		void IEditableCollectionView.Remove(object item)
		{
			this.Remove(item as TEntity);
		}

		/// <summary>Begins an edit transaction on the specified item.</summary>
		/// <param name="item">The item to edit.</param>
		public void EditItem(object item)
		{
			if (!(item is TEntity))
			{
				throw new ArgumentException("Item must inherit from " + typeof(TEntity).Name, "item");
			}

			if (this.IsEditingItem)
			{
				this.CommitEdit();
			}

			var editingEventArgs = new CancelableEntityEventArgs<TEntity>{Entity = (TEntity) item};
			this.OnEditingItem(editingEventArgs);

			if (editingEventArgs.Cancel)
			{
				return;
			}

			this.CurrentEditItem = editingEventArgs.Entity;
			((IEditableObject) this.CurrentEditItem).BeginEdit();

			var editedEventArgs = new EntityEventArgs<TEntity>(this.CurrentEditItem);
			this.OnEditedItem(editedEventArgs);
		}

		/// <summary>Ends the edit transaction and saves the pending changes.</summary>
		public void CommitEdit()
		{
			if (!this.IsEditingItem)
			{
				return;
			}

			var committingEditEventArgs = new CancelEventArgs();
			this.OnCommittingEdit(committingEditEventArgs);

			if (committingEditEventArgs.Cancel)
			{
				return;
			}

			var currentEditItem = this.CurrentEditItem;

			((IEditableObject) this.CurrentEditItem).EndEdit();
			this.CurrentEditItem = null;

			var committedEditEventArgs = new EntityEventArgs<TEntity>(currentEditItem);
			this.OnCommittedEdit(committedEditEventArgs);
		}

		/// <summary>Ends the edit transaction and, if possible, restores the original value
		/// of the item.</summary>
		public void CancelEdit()
		{
			if (!this.IsEditingItem)
			{
				return;
			}

			var cancellingEditEventArgs = new CancelEventArgs();
			this.OnCancellingNew(cancellingEditEventArgs);

			if (cancellingEditEventArgs.Cancel)
			{
				return;
			}

			var currentEditItem = this.CurrentEditItem;

			((IEditableObject) this.CurrentEditItem).CancelEdit();
			this.CurrentEditItem = null;

			
			var cancelledEditEventArgs = new EntityEventArgs<TEntity>(currentEditItem);
			this.OnCancelledNew(cancelledEditEventArgs);
		}

		/// <summary>Gets or sets the position of the new item placeholder in the collection
		/// view.</summary>
		/// <returns>An enumeration value that specifies the position of the new item placeholder
		/// in the collection view.</returns>
		public NewItemPlaceholderPosition NewItemPlaceholderPosition { get; set; }

		/// <summary>Gets a value that indicates whether a new item can be added to the collection.
		/// </summary>
		/// <returns>true if a new item can be added to the collection; otherwise, false.
		/// </returns>
		public bool CanAddNew
		{
			get
			{
				return this.delegates.CanAdd();
			}
		}

		/// <summary>Gets a value that indicates whether an add transaction is in progress.
		/// </summary>
		/// <returns>true if an add transaction is in progress; otherwise, false.</returns>
		public bool IsAddingNew
		{
			get
			{
				return this.CurrentAddItem != null;
			}
		}

		/// <summary>Gets the item that is being added during the current add transaction.
		/// </summary>
		/// <returns>The item that is being added if <see cref="P:System.ComponentModel.IEditableCollectionView.IsAddingNew"/>
		/// is true; otherwise, null.</returns>
		public TEntity CurrentAddItem { get; private set; }

		object IEditableCollectionView.CurrentAddItem
		{
			get
			{
				return this.CurrentAddItem;
			}
		}

		/// <summary>Gets a value that indicates whether an item can be removed from the
		/// collection.</summary>
		/// <returns>true if an item can be removed from the collection; otherwise, false.
		/// </returns>
		public bool CanRemove
		{
			get
			{
				return this.delegates.CanRemove();
			}
		}

		/// <summary>Gets a value that indicates whether the collection view can discard
		/// pending changes and restore the original values of an edited object.</summary>
		/// <returns>true if the collection view can discard pending changes and restore
		/// the original values of an edited object; otherwise, false.</returns>
		public bool CanCancelEdit
		{
			get
			{
				return true;
			}
		}

		/// <summary>Gets a value that indicates whether an edit transaction is in progress.
		/// </summary>
		/// <returns>true if an edit transaction is in progress; otherwise, false.</returns>
		public bool IsEditingItem
		{
			get
			{
				return this.CurrentEditItem != null;
			}
		}

		/// <summary>Gets the item in the collection that is being edited.</summary>
		/// <returns>The item that is being edited if <see cref="P:System.ComponentModel.IEditableCollectionView.IsEditingItem"/>
		/// is true; otherwise, null.</returns>
		public TEntity CurrentEditItem { get; private set; }

		object IEditableCollectionView.CurrentEditItem
		{
			get
			{
				return this.CurrentEditItem;
			}
		}

		/// <summary>Removes the item at the specified position from the collection.</summary>
		/// <param name="index">Index of item to remove.</param>
		void IEditableCollectionView.RemoveAt(int index)
		{
			this.RemoveAt(index);
		}
	}
}
