﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace DeleteFromEntitySet
{
	public partial class EntityCollectionViewBase<TEntity> : IList<TEntity>
	{
		/// <summary>Returns an enumerator that iterates through the collection.</summary>
		/// <returns>A <see cref="T:System.Collections.Generic.IEnumerator`1"/> that can
		/// be used to iterate through the collection.</returns>
		public IEnumerator<TEntity> GetEnumerator()
		{
			return this.entities.GetEnumerator();
		}

		/// <summary>Returns an enumerator that iterates through a collection.</summary>
		/// <returns>An <see cref="T:System.Collections.IEnumerator"/> object that can be
		/// used to iterate through the collection.</returns>
		IEnumerator IEnumerable.GetEnumerator()
		{
			return this.GetEnumerator();
		}

		/// <summary>Defines methods to manipulate generic collections.</summary>
		/// <typeparam name="T">The type of the elements in the collection.</typeparam>
		public void Add(TEntity item)
		{
			this.delegates.Add(item);
		}

		/// <summary>Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
		/// </summary>
		/// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/>
		/// is read-only. </exception>
		public void Clear()
		{
			foreach (var removedEntity in this.ToList())
			{
				this.Remove(removedEntity);
			}
		}

		/// <summary>Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
		/// </summary>
		/// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/>
		/// is read-only. </exception>
		public bool Contains(TEntity item)
		{
			return this.entities.Contains(item);
		}

		/// <summary>Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
		/// </summary>
		/// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/>
		/// is read-only. </exception>
		void ICollection<TEntity>.CopyTo(TEntity[] array, int arrayIndex)
		{
			// TODO: Implement this method
			throw new NotImplementedException();
		}

		/// <summary>Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
		/// </summary>
		/// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/>
		/// is read-only. </exception>
		public bool Remove(TEntity item)
		{
			if (this.Contains(item))
			{
				var addingEventArgs = new CancelableEntityEventArgs<TEntity>{ Entity = item };
				this.OnRemoving(addingEventArgs);

				if (!addingEventArgs.Cancel)
				{
					this.delegates.Remove(addingEventArgs.Entity);

					this.OnRemoved(new EntityEventArgs<TEntity>(addingEventArgs.Entity));

					return true;
				}
			}
			return false;
		}

		/// <summary>Gets the number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.
		/// </summary>
		/// <returns>The number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.
		/// </returns>
		public int Count
		{
			get
			{
				return this.delegates.Count();
			}
		}

		/// <summary>Gets a value indicating whether the <see cref="T:System.Collections.Generic.ICollection`1"/>
		/// is read-only.</summary>
		/// <returns>true if the <see cref="T:System.Collections.Generic.ICollection`1"/>
		/// is read-only; otherwise, false.</returns>
		public bool IsReadOnly
		{
			get
			{
				return !(this.delegates.CanAdd() && this.delegates.CanRemove());
			}
		}

		/// <summary>Represents a collection of objects that can be individually accessed
		/// by index.</summary>
		/// <typeparam name="T">The type of elements in the list.</typeparam>
		public int IndexOf(TEntity item)
		{
			int index = -1;
			foreach (var entity in this.entities)
			{
				index++;
				if (entity == item)
				{
					return index;
				}
			}

			return index;
		}

		/// <summary>Represents a collection of objects that can be individually accessed
		/// by index.</summary>
		/// <typeparam name="T">The type of elements in the list.</typeparam>
		public void Insert(int index, TEntity item)
		{
			this.delegates.Add(item);
		}

		/// <summary>Removes the <see cref="T:System.Collections.Generic.IList`1"/> item
		/// at the specified index.</summary>
		/// <param name="index">The zero-based index of the item to remove.</param>
		/// <exception cref="T:System.ArgumentOutOfRangeException">
		/// <paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.
		/// </exception>
		/// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.IList`1"/>
		/// is read-only.</exception>
		public void RemoveAt(int index)
		{
			var removedEntity = this.entities.ElementAt(index);
			if (removedEntity != null)
			{
				this.delegates.Remove(removedEntity);
			}
		}

		/// <summary>Removes the <see cref="T:System.Collections.Generic.IList`1"/> item
		/// at the specified index.</summary>
		/// <param name="index">The zero-based index of the item to remove.</param>
		/// <exception cref="T:System.ArgumentOutOfRangeException">
		/// <paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.
		/// </exception>
		/// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.IList`1"/>
		/// is read-only.</exception>
		public TEntity this[int index]
		{
			get
			{
				return this.entities.ElementAt(index);
			}
			set
			{
				throw new NotImplementedException();
			}
		}
	}
}
